package com.style24.scm.biz.dao;

import java.util.Collection;

import com.gagaframework.web.parameter.GagaMap;
import com.style24.core.support.annotation.ShopDs;
import com.style24.persistence.domain.Pos;
import com.style24.persistence.domain.Erp;

/**
 * 매장POS Dao
 *
 * @author moon
 * @since 2021. 06. 11
 */
@ShopDs
public interface TssPosDao {

	
	/**
	 * 매장로그인정보 조회
	 * @param Pos
	 * @return
	 * @author moon
	 * @since 2021. 06. 11
	 */
	Pos getStoreLoginInfo(Pos pos);
	
	/**
	 * 출고대기목록, 출고목록
	 * 
	 * @param Pos - 매장
	 * @return Collection<Pos>
	 * @author moon
	 * @since 2021. 06. 11
	 */
	Collection<Pos> getPosDeliveryList(Pos pos);
	
	/**
	 * 확정대기목록
	 * 
	 * @param Pos - 매장
	 * @return Collection<Pos>
	 * @author moon
	 * @since 2021. 06. 21
	 */
	Collection<Pos> getPosConfirmList(Pos pos);	
	
	/**
	 * 출고수락 
	 * 
	 * @param Pos - 매장
	 * @return Collection<Pos>
	 * @author moon
	 * @since 2021. 06. 13
	 */	
	int updateDeliveryAssign(Pos pos);
	
	/**
	 * 출고 이력등록
	 * 
	 * @param Pos - 매장
	 * @return Collection<Pos>
	 * @author moon
	 * @since 2021. 06. 13
	 */	
	int createDeliveryAssignHst(Pos pos);	
	
	/**
	 * 주문상세상태 변경
	 * 
	 * @param Pos - 매장
	 * @return Collection<Pos>
	 * @author moon
	 * @since 2021. 06. 13
	 */	
	int updateOrderDetail(Pos pos);
	
	
	/**
	 * 출고금지상품 등록
	 *
	 * @param Pos
	 * @author moon
	 * @since 2021. 06. 13
	 */
	void createBangoods(Pos pos);
	
	/**
	 * 주문상세아이템리스트 
	 * 
	 * @param Pos - 매장
	 * @return Collection<Pos>
	 * @author moon
	 * @since 2021. 06. 13
	 */
	Collection<Pos> getOrdDtlItemList(Pos pos);
	
	
	/**
	 * 출고처 체크 
	 * 
	 * @param Pos - 매장
	 * @return Pos
	 * @author moon
	 * @since 2021. 06. 14
	 */
	Pos getDelvLocChk(Pos pos);
	
	/**
	 * 주문상세 상태 조회 
	 * 
	 * @param Pos - 매장
	 * @return Pos
	 * @author moon
	 * @since 2021. 06. 14
	 */	
	Pos getOrderDtlStat(Pos pos);
	
	/**
	 * Cnplus엑셀다운로드
	 * 
	 * @param pos - 매장
	 * @return Collection<TsaPos>
	 * @author moon
	 * @since 2021. 06. 14
	 */
	Collection<GagaMap> getOrderExcelList(Pos pos);	
	
	/**
	 * 송장번호 등록
	 * 
	 * @param Pos - 매장
	 * @return Collection<Pos>
	 * @author moon
	 * @since 2021. 06. 21
	 */	
	int insertTbOderDetailInvoice(Pos pos);

	/**
	 * 출고처지정 취소
	 * 
	 * @param Pos - 매장
	 * @return Collection<Pos>
	 * @author moon
	 * @since 2021. 06. 21
	 */	
	int updateDeliveryAssignCancel(Pos pos);	
	
	/**
	 * ERP - 주문상세아이템리스트 
	 * 
	 * @param Pos - 매장
	 * @return Collection<Pos>
	 * @author moon
	 * @since 2021. 06. 21
	 */
	Collection<Pos> getSendErpList(Pos pos);	
	
	/**
	 * 매장 POS 정산내역
	 * 
	 * @param pos - 매장
	 * @return Collection<TsaPos>
	 * @author moon
	 * @since 2021. 06. 21
	 */
	Collection<Pos> getPosUsacList(Pos pos);	
	
	/**
	 * 매장거부 목록
	 *
	 * @param Pos
	 * @return Collection<Pos>
	 * @author moon
	 * @since 2021. 06. 21
	 */
	Collection<Pos> getDeliveryAssignList(Pos pos);	
}
